<?php

namespace seasar\log;
interface LoggerFactory {
    
    public function getInstance($className);
}

namespace seasar\container;
interface S2Container {
    
    public function getComponent($componentKey);
    
    public function findComponents($componentKey);
    
    public function register($component, $componentName = "");
    
    public function getComponentDefSize();
    
    public function getComponentDefNames();
    
    public function getComponentDef($index);
    
    public function findComponentDefs($componentKey);
    
    public function hasComponentDef($componentKey);
    
    public function hasDescendant($path);
        
    public function getDescendant($path);
    
    public function registerDescendant(S2Container $descendant);
    
    public function includeChild(S2Container $child);
    
    public function getChildSize();
    
    public function getChild($index);
        
    public function getNamespace();
        
    public function setNamespace($namespace);
        
    public function getPath();
        
    public function setPath($path);
    
    public function getRoot();
        
    public function setRoot(S2Container $root);
}

namespace seasar\container;
interface ComponentDef {
    
    public function getComponent();
    
    public function getContainer();
    
    public function setContainer(\seasar\container\S2Container $container);
    
    public function getComponentClass();
    
    public function getComponentName();
    
    public function setComponentName($name);
    
    public function getAutoBindingDef();
    
    public function setAutoBindingDef(\seasar\container\AutoBindingDef $def);
    
    public function getInstanceDef();
    
    public function setInstanceDef(\seasar\container\InstanceDef $def);
    
    public function getArgDefs();
    
    public function getArgDef($index);
    
    public function getArgDefSize();
    
    public function addArgDef(\seasar\container\impl\ArgDef $argDef);
    
    public function getPropertyDefs();
    
    public function getPropertyDef($index);
    
    public function hasPropertyDef($propertyName);
    
    public function addPropertyDef(\seasar\container\impl\PropertyDef $propertyDef);
    
    public function getPropertyDefSize();
    
    public function getInitMethodDefs();
    
    public function getInitMethodDef($index);
    
    public function getInitMethodDefSize();
    
    public function addInitMethodDef(\seasar\container\impl\InitMethodDef $methodDef);
    
    public function getAspectDefs();
    
    public function getAspectDef($index);
    
    public function getAspectDefSize();
    
    public function addAspectDef(\seasar\container\impl\AspectDef $aspectDef);
    
    public function getMetaDef($index);
    
    public function getMetaDefs($name);
    
    public function addMetaDef(\seasar\container\impl\MetaDef $metaDef);
    
    public function getMetaDefSize();
    
    public function setConstructClosure(\Closure $closure);
    
    public function getConstructClosure();
    
    public function hasConstructClosure();
}

namespace seasar\container;
interface InstanceDef {
    
    const SINGLETON_NAME = 'singleton';
    
    const PROTOTYPE_NAME = 'prototype';
    
    public function getName();
    
    public function createComponentDeployer(\seasar\container\ComponentDef $componentDef);
}

namespace seasar\container;
interface AutoBindingDef {
    
    const AUTO_NAME = 'auto';
   
    const NONE_NAME = 'none';
    
    public function getName();
    
    public function createConstructorAssembler(\seasar\container\ComponentDef $componentDef);
    
    public function createPropertyAssembler(\seasar\container\ComponentDef $componentDef);
}

namespace seasar\beans;
interface PropertyDesc {
    
    public function setValue($instance, $value);
    
    public function getValue($instance);
    
    public function getReflection();
    
    public function hasReadMethod();
    
    public function hasWriteMethod();
    
    public function isReadable();
    
    public function isWritable();
    
    public function getBeanClass();
    
    public function getPropertyName();
    
    public function getTypehint();
    
    public function setTypehint($value);
    
    public function getTypehintClass();
    
    public function setTypehintClass(\ReflectionClass $clazz);
    
    public function isArrayAcceptable();
    
    public function setArrayAcceptable($value = true);
}

namespace seasar\exception;
class S2RuntimeException extends \RuntimeException {
    
    public static $MESSAGES = array();
    
    protected $code  = null;
    
    private $args  = null;
    
    private $couse = null;
    
    public function __construct($code, array $args = array(), \Exception $couse = null) {
        $this->code  = $code;
        $this->args  = $args;
        $this->couse = $couse;
        if (isset(self::$MESSAGES[$code])) {
            $msg = \seasar\util\EvalUtil::formatExecute(self::$MESSAGES[$code], array('args' => $args));
        } else {
            $msgTmp = array();
            foreach ($args as $arg) {
                $msgTmp[] = \seasar\util\StringUtil::mixToString($arg);
            }
            $msg = '[' . implode(', ', $msgTmp) . ']';
        }
        parent::__construct($msg, $code);
    }
    
    public function getArgs() {
        return $this->args;
    }
    
    public function getCouse() {
        return $this->couse;
    }
}

namespace seasar\container;
class ComponentInfoDef {
    public static $USE_PHP_NAMESPACE = false;
    private $reflectionClass = null;
    private $className = null;
    private $name = null;
    private $instance = null;
    private $autoBinding = null;
    private $namespace = null;
    private $usePhpNamespace = null;
    private $constructClosure = null;
    
    public function __construct($clazz = null) {
        if ($clazz instanceof \ReflectionClass) {
            $this->reflectionClass = $clazz;
            $this->className = $clazz->getName();
        } else {
            $this->className = $clazz;
        }
        $this->usePhpNamespace = self::$USE_PHP_NAMESPACE;
    }
    
    public function setReflectionClass($calzz) {
        $this->reflectionClass = $clazz;
        $this->className = $clazz->getName();
        return $this;
    }
    
    public function getReflectionClass() {
        if (is_null($this->reflectionClass)) {
            $this->reflectionClass = new \ReflectionClass($this->className);
        }
        return $this->reflectionClass;
    }
    
    public function setClassName($className) {
        $this->className = $className;
        return $this;
    }
    
    public function getClassName() {
        return $this->className;
    }
    
    public function hasName() {
        return !is_null($this->name);
    }
    
    public function setName($name) {
        $this->name = $name;
        return $this;
    }
    
    public function getName() {
        return $this->name;
    }
    
    public function hasInstance() {
        return !is_null($this->instance);
    }
    
    public function setInstance($instance) {
        $this->instance = $instance;
        return $this;
    }
    
    public function getInstance() {
        return $this->instance;
    }
    
    public function hasAutoBinding() {
        return !is_null($this->autoBinding);
    }
    
    public function setAutoBinding($autoBinding) {
        $this->autoBinding = $autoBinding;
        return $this;
    }
    
    public function getAutoBinding() {
        return $this->autoBinding;
    }
    
    public function hasNamespace() {
        return !is_null($this->namespace);
    }
    
    public function setNamespace($namespace) {
        $this->namespace = $namespace;
        return $this;
    }
    
    public function getNamespace() {
        if (is_null($this->namespace)) {
            if ($this->usePhpNamespace === true) {
                $ns = $this->getReflectionClass()->getNamespaceName();
                $this->namespace = str_replace('\\', '.', $ns);
            }
        }
        return $this->namespace;
    }
    
    public function usePhpNamespace($val = true) {
        $this->usePhpNamespace = $val;
        return $this;
    }
    
    public function isUsePhpNamespace() {
        return $this->usePhpNamespace;
    }
    
    public function setConstructClosure(\Closure $constructClosure) {
        $this->constructClosure = $constructClosure;
        return $this;
    }
    
    public function construct(\Closure $constructClosure) {
        return $this->setConstructClosure($constructClosure);
    }
    
    public function getConstructClosure() {
        return $this->constructClosure;
    }
    
    public function hasConstructClosure() {
        return !is_null($this->constructClosure);
    }
}

namespace seasar\container;
class S2ApplicationContext {
    
    const DEFAULT_SINGLETON_CONTAINER_KEY = '';
    
    public static $CLASSES = array();
    
    public static $DICONS = array();
    
    public static $COMPONENT_INFOS = array();
    
    public static $includeDiconPatterns = array();
    
    public static $excludeDiconPatterns = array();
    
    public static $includeClassPatterns = array();
    
    public static $excludeClassPatterns = array();
    
    public static $autoAspects = array();
    
    public static $SINGLETON_CONTAINERS = array();
    
    public static function init() {
        self::$CLASSES = array();
        self::$DICONS  = array();
        self::$COMPONENT_INFOS = array();
        self::$SINGLETON_CONTAINERS = array();
        self::$includeDiconPatterns = array();
        self::$excludeDiconPatterns = array();
        self::$includeClassPatterns = array();
        self::$excludeClassPatterns = array();
        self::$autoAspects = array();
    }
    
    public static function setReadParentAnnotation($val = true) {
        \seasar\container\factory\ComponentDefBuilder::$READ_PARENT_ANNOTATION = $val;
    }
    
    public static function add($info) {
        return self::register($info);
    }
    
    public static function register($clazz = null) {
        if ($clazz instanceof seasar\container\ComponentInfoDef) {
            $info = $clazz;
        } else {
            $info = new \seasar\container\ComponentInfoDef($clazz);
        }
        self::$COMPONENT_INFOS[] = $info;
        return $info;
    }
    
    public static function registerAspect($interceptor = null, $componentPattern = null, $pointcut = null) {
        if ($interceptor instanceof \seasar\container\AspectInfoDef) {
            $info = $interceptor;
        } else {
            $info = new \seasar\container\AspectInfoDef($interceptor, $componentPattern, $pointcut);
        }
        self::$autoAspects[] = $info;
        return $info;
    }
    
    public static function import($path, $namespace = array(), $strict = false, $pear = false, $recursive = true) {
        if (is_string($namespace)) {
            if ($pear) {
                $namespace = explode('_', $namespace);
            } else {
                $namespace = explode('\\', $namespace);
            }
        }
        if (is_dir($path)) {
            self::scanDir($path, $namespace, $strict, $pear, $recursive);
        } else if (is_file($path)) {
            self::importInternal($path, $namespace, $pear);
        } else if (class_exists($path) or interface_exists($path)) {
            self::$CLASSES[$path] = null;
        } else {
            throw new \seasar\exception\FileNotFoundException($path);
        }
    }
    
    private static function scanDir($parentPath, $namespace, $strict, $pear, $recursive) {
        $iterator = new \DirectoryIterator($parentPath);
        while($iterator->valid()) {
            if (strpos($iterator->getFilename(), '.') === 0) {
                $iterator->next();
                continue;
            }
            if ($iterator->isFile()) {
                self::importInternal($iterator->getRealPath(), $namespace, $pear);
            } else if ($recursive and $iterator->isDir()) {
                if ($strict) {
                    self::scanDir($iterator->getRealPath(), $namespace, $strict, $pear, $recursive);
                } else {
                    self::scanDir($iterator->getRealPath(), array_merge($namespace, (array)$iterator->getFileName()), $strict, $pear, $recursive);
                }
            }
            $iterator->next();
        }
    }
    
    public static function importInternal($filePath, array $namespace = array(), $pear = false){
        $fileName = basename($filePath);
        $fileNameRev = strrev($fileName);
        if (stripos($fileNameRev, 'php.') === 0) {
            $namespace[] = substr($fileName, 0, strpos($fileName, '.'));
            if ($pear) {
                $className = implode('_' , $namespace);
            } else {
                $className = implode('\\' , $namespace);
            }
            self::$CLASSES[$className] = $filePath;
            \seasar\util\ClassLoader::$CLASSES[$className] = $filePath;
            \seasar\log\S2Logger::getLogger(__CLASS__)->debug("find class $className : $filePath", __METHOD__);
        } else if (stripos($fileNameRev, 'nocid.') === 0) {
            self::$DICONS[$fileName] = $filePath;
            \seasar\log\S2Logger::getLogger(__CLASS__)->debug("find dicon $fileName : $filePath", __METHOD__);
        } else {
            \seasar\log\S2Logger::getLogger(__CLASS__)->debug("ignore file $fileName : $filePath", __METHOD__);
        }
    }
    
    public static function getComponent($key, $namespaces = array()) {
        return self::getComponentDef($key, $namespaces)->getComponent();
    }
    
    public static function get($key, $namespaces = array()) {
        return self::getComponent($key, $namespaces);
    }
    
    public static function getComponentDef($key, $namespaces = array()) {
        return self::createSingletonContainer($namespaces)->getComponentDef($key);
    }
    
    public static function hasComponentDef($key, $namespaces = array()) {
        return self::createSingletonContainer($namespaces)->hasComponentDef($key);
    }
    
    public static function createSingletonContainer($namespaces = array(), $force = false) {
        $namespaces = (array)$namespaces;
        if (count($namespaces) === 0) {
            $singletonKey = self::DEFAULT_SINGLETON_CONTAINER_KEY;
        } else {
            $singletonKey = self::createSingletonKey($namespaces);
        }
        if (!array_key_exists($singletonKey, self::$SINGLETON_CONTAINERS) || $force) {
            self::$SINGLETON_CONTAINERS[$singletonKey] = self::create($namespaces);
        }
        return self::$SINGLETON_CONTAINERS[$singletonKey];
    }
    
    public static function createSingletonKey($namespaces = array()) {
        $namespaces = (array)$namespaces;
        sort($namespaces, SORT_STRING);
        return implode(',', $namespaces);
    }
    
    public static function create($namespaces = array()) {
        $namespaces = (array)$namespaces;
        $dicons = array_values(self::$DICONS);
        if (count($dicons) > 0) {
            $dicons = self::includeFilter($dicons, self::$includeDiconPatterns);
            $dicons = self::excludeFilter($dicons, self::$excludeDiconPatterns);
        }
        $classes = array_keys(self::$CLASSES);
        if (count($classes) > 0) {
            $classes = self::includeFilter($classes, self::$includeClassPatterns);
            $classes = self::excludeFilter($classes, self::$excludeClassPatterns);
        }
        $infos = self::$COMPONENT_INFOS;
        foreach ($classes as $className) {
            $info = self::createComponentInfoDef($className);
            if (!is_null($info)) {  // if available of @S2Component annotation is flase
                $infos[] = $info;
            }
        }
        if (count($dicons) == 0 and count($infos) == 0) {
            \seasar\log\S2Logger::getLogger(__CLASS__)->info("dicon, class not found at all. create empty container.", __METHOD__);
            return new \seasar\container\impl\S2ContainerImpl();
        }
        $container = self::createInternal($dicons, $infos, $namespaces);
        return $container;
    }
    
    public static function createComponentInfoDef($className) {
        $refClass = new \ReflectionClass($className);
        if (!\seasar\util\Annotation::has($refClass, \seasar\container\factory\ComponentDefBuilder::COMPONENT_ANNOTATION)) {
            return new \seasar\container\ComponentInfoDef($refClass);
        }
        $componentInfo = \seasar\util\Annotation::get($refClass, \seasar\container\factory\ComponentDefBuilder::COMPONENT_ANNOTATION);
        if (isset($componentInfo['available']) and (boolean)$componentInfo['available'] === false) {
            return null;
        }
        $info = new \seasar\container\ComponentInfoDef($refClass);
        if (isset($componentInfo['name'])) {
            $info->setName($componentInfo['name']);
        }
        if (isset($componentInfo['instance'])) {
            $info->setInstance($componentInfo['instance']);
        }
        if (isset($componentInfo['autoBinding'])) {
            $info->setAutoBinding($componentInfo['autoBinding']);
        }
        if (isset($componentInfo['namespace'])) {
            $info->setNamespace($componentInfo['namespace']);
        }
        return $info;
    }
    
    public static function createInternal($dicons, $infos, $namespaces) {
        $container = new \seasar\container\impl\S2ContainerImpl();
        foreach ($dicons as $dicon) {
            $child = \seasar\container\factory\S2ContainerFactory::includeChild($container, $dicon);
            $child->setRoot($container->getRoot());
            \seasar\log\S2Logger::getLogger(__CLASS__)->debug("include dicon : $dicon", __METHOD__);
        }
        $registeredComponentDefs = array();
        $checkNamespace = false;
        $dotNamespaces = array();
        if (0 < count($namespaces)) {
            $checkNamespace = true;
            foreach($namespaces as $n) {
                $dotNamespaces[] = $n . '.';
            }
        } 
        foreach($infos as $info) {
            if ($checkNamespace) {
                $namespace = (string)$info->getNamespace();
                if (!in_array($namespace, $namespaces, true)) {
                    $applicable = false;
                    foreach($dotNamespaces as $n) {
                        if (0 === strpos($namespace, $n)) {
                            $applicable = true;
                            break;
                        }
                    }
                    if (!$applicable) {
                        $nsStr = implode("', '", $namespaces);
                        \seasar\log\S2Logger::getLogger(__CLASS__)->debug($info->getClassName() . " ignored by namespace : '$namespace' not in array('$nsStr')", __METHOD__);
                        continue;
                    }
                }
            }
            $cd = self::createComponentDef($info);
            $registeredComponentDefs[] = $cd;
            self::registerComponentDef($container, $cd, (string)$info->getNamespace());
            \seasar\log\S2Logger::getLogger(__CLASS__)->debug('import component : ' . $info->getClassName(), __METHOD__);
        }
        \seasar\log\S2Logger::getLogger(__CLASS__)->info('imported components : ' . count($registeredComponentDefs), __METHOD__);
        foreach($registeredComponentDefs as $cd) {
            \seasar\container\factory\ComponentDefBuilder::setupComponentDef($cd);
            foreach(self::$autoAspects as $aspectInfo) {
                if ($aspectInfo->applicable($cd)) {
                    \seasar\container\factory\ComponentDefBuilder::setupAspectDef($cd, $aspectInfo->toAnnotationArray());
                }
            }
        }
        return $container;
    }
    
    public static function createComponentDef($info) {
        if ($info->hasName()) {
            $cd = new \seasar\container\impl\ComponentDefImpl($info->getReflectionClass(), $info->getName());
        } else {
            $cd = new \seasar\container\impl\ComponentDefImpl($info->getReflectionClass());
        }
        if ($info->hasInstance()) {
            $cd->setInstanceDef(\seasar\container\deployer\InstanceDefFactory::getInstanceDef($info->getInstance()));
        }
        if ($info->hasAutoBinding()) {
            $cd->setAutoBindingDef(\seasar\container\assembler\AutoBindingDefFactory::getAutoBindingDef($info->getAutoBinding()));
        }
        if ($info->hasConstructClosure()) {
            $cd->setConstructClosure($info->getConstructClosure());
        }
        return $cd;
    }
    
    public static function registerComponentDef(\seasar\container\S2Container $container, \seasar\container\ComponentDef $cd, $namespace = '') {
        if ($namespace == '') { // 文字列一致、またはnull一致
            $container->register($cd);
        } else {
            $names = preg_split('/\./', $namespace, 2);
            if ($container->hasComponentDef($names[0])) {
                $childContainer = $container->getComponent($names[0]);
                if (!$childContainer instanceof \seasar\container\S2Container) {
                    throw new \seasar\container\exception\TooManyRegistrationRuntimeException($names[0], array($container->getComponentDef($names[0])->getComponentClass(), new \ReflectionClass('\seasar\container\impl\S2ContainerImpl')));
                }
            } else {
                $childContainer = new \seasar\container\impl\S2ContainerImpl();
                $childContainer->setNamespace($names[0]);
                $container->includeChild($childContainer);
            }
            $restNamespace = '';
            if (count($names) == 2) {
                $restNamespace = $names[1];
            }
            self::registerComponentDef($childContainer, $cd, $restNamespace);
        }
    }
    
    public static function includeFilter($items, $patterns) {
        if (count($patterns) == 0) {
            return $items;
        }
        $results = array();
        foreach ($items as $item) {
            foreach ($patterns as $pattern) {
                if (preg_match($pattern, $item)) {
                    $results[] = $item;
                    break;
                }
            }
        }
        return $results;
    }
    
    public static function excludeFilter($items, $patterns) {
        if (count($patterns) == 0) {
            return $items;
        }
        $results = array();
        foreach ($items as $item) {
            $exclude = false;
            foreach ($patterns as $pattern) {
                if (preg_match($pattern, $item)) {
                    $exclude = true;
                    break;
                }
            }
            if ($exclude == false) {
                $results[] = $item;
            }
        }
        return $results;
    }
    
    public static function setPattern($pattern, $condition = true) {
        if ($condition) {
            self::setIncludePattern($pattern);
        } else {
            self::setExcludePattern($pattern);
        }
    }
    
    public static function addPattern($pattern, $condition = true) {
        if ($condition) {
            self::addIncludePattern($pattern);
        } else {
            self::addExcludePattern($pattern);
        }
    }
    
    public static function setIncludePattern($pattern = array()) {
        self::$includeDiconPatterns = (array)$pattern;
        self::$includeClassPatterns = (array)$pattern;
    }
    
    public static function addIncludePattern($pattern) {
        self::$includeDiconPatterns[] = $pattern;
        self::$includeClassPatterns[] = $pattern;
    }
    
    public static function setExcludePattern($pattern = array()) {
        self::$excludeDiconPatterns = (array)$pattern;
        self::$excludeClassPatterns = (array)$pattern;
    }
    
    public static function addExcludePattern($pattern) {
        self::$excludeDiconPatterns[] = $pattern;
        self::$excludeClassPatterns[] = $pattern;
    }
}

namespace seasar\log;
class S2Logger {
    
    private static $loggerFactory = null;
    
    private function __construct() {}
    
    public static function setLoggerFactory(\seasar\log\LoggerFactory $factory) {
        self::$loggerFactory = $factory;
    }
    
    public static function getInstance($className = null) {
        return self::getLogger($className);
    }
    
    public static function getLogger($className = null) {
        return self::getLoggerFactory()->getInstance($className);
    }
    
    public static function getLoggerFactory() {
        if (self::$loggerFactory === null) {
            self::$loggerFactory = new \seasar\Config::$LOGGER_FACTORY;
        }
        return self::$loggerFactory;
    }
}

namespace seasar\log\impl;
class SimpleLogger {
    const DEBUG  = 1;
    const INFO   = 2;
    const NOTICE = 3;
    const WARN   = 4;
    const ERROR  = 5;
    const FATAL  = 6;
    
    private $labels = array(
                          self::DEBUG  => 'DEBUG',
                          self::INFO   => 'INFO',
                          self::NOTICE => 'NOTICE',
                          self::WARN   => 'WARN',
                          self::ERROR  => 'ERROR',
                          self::FATAL  => 'FATAL');
    
    public function debug($msg = '-', $methodName = '-') {
        $this->logging(self::DEBUG, $msg, $methodName);
    }
    
    public function info($msg = '-', $methodName = '-') {
        $this->logging(self::INFO, $msg, $methodName);
    }
    
    public function notice($msg = '-', $methodName = '-') {
        $this->logging(self::NOTICE, $msg, $methodName);
    }
    
    public function warn($msg = '-', $methodName = '-') {
        $this->logging(self::WARN, $msg, $methodName);
    }
    
    public function error($msg = '-', $methodName = '-') {
        $this->logging(self::ERROR, $msg, $methodName);
    }
    
    public function fatal($msg = '-', $methodName = '-') {
        $this->logging(self::FATAL, $msg, $methodName);
    }
    
    private function logging($level, $msg, $methodName) {
        if (\seasar\Config::$LOG_LEVEL <= $level) {
            $logMsg = sprintf('%s [%-6s] %s - %s' . PHP_EOL, date('Y-m-d H:i:s'), $this->labels[$level], $methodName, $msg);
            if (\seasar\Config::$SIMPLE_LOG_FILE !== null) {
                file_put_contents(\seasar\Config::$SIMPLE_LOG_FILE, $logMsg, FILE_APPEND | LOCK_EX);
            } else {
                print $logMsg;
            }
        }
    }
}

namespace seasar\log\impl;
class SimpleLoggerFactory implements \seasar\log\LoggerFactory {
    
    private $logger = null;
    
    public function getInstance($className) {
        if ($this->logger === null) {
            $this->logger = new SimpleLogger();
        }
        return $this->logger;
    }
}

namespace seasar\container\impl;
class S2ContainerImpl implements \seasar\container\S2Container {
    
    private $componentDefMap = array();
    
    private $componentDefList = array();
    
    private $parents = array();
    
    private $children = array();
    
    private $descendants = array();
    
    private $namespace = null;
    
    private $metaDefSupport = null;
    
    private $path = null;
    
    private $root = null;
    
    public function __construct() {
        $this->metaDefSupport = new \seasar\container\util\MetaDefSupport();
        $this->root = $this;
        $componentDef = new SimpleComponentDef($this, \seasar\container\Config::CONTAINER_NAME);
        $this->componentDefMap[\seasar\container\Config::CONTAINER_NAME] = $componentDef;
        $this->componentDefMap['seasar\container\S2Container'] = $componentDef;
    }
    
    public function getRoot() {
        return $this->root;
    }
    
    public function setRoot(\seasar\container\S2Container $root) {
        $this->root = $root;
    }
    
    public function getParents() {
        return $this->parents;
    }
    
    public function addParent(\seasar\container\S2Container $parent) {
        $this->parents[] = $parent;
    }
    
    public function getComponent($componentKey) {
        return $this->getComponentDef($componentKey)->getComponent();
    }
    
    public function findComponents($componentKey) {
        $componentDefs = $this->findComponentDefs($componentKey);
        $components = array();
        foreach ($componentDefs as $componentDef) {
            $components[] = $componentDef->getComponent();
        }
        return $components;
    }
    
    public function register($component, $componentName = '') {
        if ($component instanceof \seasar\container\ComponentDef) {
            $this->registerInternal($component);
            $this->componentDefList[] = $component;
        } else if (is_object($component)) {
            $this->register(new SimpleComponentDef($component, $componentName));
        } else {
            $this->register(new ComponentDefImpl($component, $componentName));
        }
    }
    
    private function registerInternal(\seasar\container\ComponentDef $componentDef) {
        if ($componentDef->getContainer() === null) {
            $componentDef->setContainer($this);
        }
        $this->registerByClass($componentDef);
        $this->registerByName($componentDef);
    }
    
    private function registerByClass(\seasar\container\ComponentDef $componentDef) {
        $classes = $this->getAssignableClasses($componentDef->getComponentClass());
        $componentName = $componentDef->getComponentName();
        foreach ($classes as $classInfo) {
            $className = $classInfo[1];
            $namespacedClassName = $classInfo[0] . '\\' . $className;
            if ($namespacedClassName !== $componentName) {
                $this->registerMap($namespacedClassName, $componentDef);
                if ($className !== $namespacedClassName and $className !== $componentName) {
                    $this->registerMap($className, $componentDef);
                }
                $lcClassName = lcfirst($className);
                if ($lcClassName !== $className and $lcClassName !== $namespacedClassName and $lcClassName !== $componentName) {
                    $this->registerMap($lcClassName, $componentDef);
                }
            }
        }
    }
    
    private function registerByName(\seasar\container\ComponentDef $componentDef) {
        $componentName = $componentDef->getComponentName();
        if ($componentName !== null) {
            $this->registerMap($componentName, $componentDef);
        }
    }
    
    private function registerMap($key, \seasar\container\ComponentDef $componentDef) {
        if (array_key_exists($key, $this->componentDefMap)) {
            $this->processTooManyRegistration($key, $componentDef);
        } else {
            $this->componentDefMap[$key] = $componentDef;
        }
    }
    
    public function getComponentDefSize() {
        return count($this->componentDefList);
    }
    
    public function getComponentDefNames() {
        return array_keys($this->componentDefMap);
    }
    
    public function getComponentDef($key) {
        if (is_int($key)) {
            if (!isset($this->componentDefList[$key])) {
                throw new \seasar\container\exception\ComponentNotFoundRuntimeException($key);
            }
            return $this->componentDefList[$key];
        }
        if (is_object($key)) {
            $key = get_class($key);
        }
        $componentDef = $this->getComponentDefInternal($key);
        if ($componentDef === null) {
            throw new \seasar\container\exception\ComponentNotFoundRuntimeException($key);
        }
        return $componentDef;
    }
    
    public function findComponentDefs($key)
    {
        $componentDef = $this->getComponentDefInternal($key);
        if ($componentDef === null) {
            return array();
        } else if ($componentDef instanceof TooManyRegistrationComponentDef) {
            return $componentDef->getComponentDefs();
        }
        return array($componentDef);
    }
    
    private function getComponentDefInternal($key) {
        if (0 === strpos($key, '\\')) {
            $key = substr($key, 1);
        }
        $cd = $this->getComponentDefRecursive($key);
        if (!is_null($cd)) {
            return $cd;
        }
        if (false === \seasar\container\Config::$AUTO_REGISTER_WHEN_NOT_FOUND) {
            return null;
        }
        if (class_exists($key) || interface_exists($key)) {
            $refClass = new \ReflectionClass($key);
            if ($refClass->isUserDefined()) {
                $cd = \seasar\container\factory\ComponentDefBuilder::create($this, $refClass);
                if ($refClass->isAbstract() || $refClass->isInterface()) {
                    if (count($cd->getAspectDefs()) === 0) {
                        return null;
                    }
                }
                $this->register($cd);
                return $cd;
            }
        }
        return null;
    }
    
    public function getComponentDefRecursive($key, $searchParent = true, &$searchedAsParent = array(), &$searchedAsChild = array()) {
        if (array_key_exists($key, $this->componentDefMap)) {
            $cd =  $this->componentDefMap[$key];
        } else if (false !== strpos($key, '.')) {
            $cd = $this->getComponentDefByNamespace($key);
        } else {
            $cd = $this->getComponentDefByChildren($key, $searchedAsParent, $searchedAsChild);
        }
        if (!is_null($cd)) {
            return $cd;
        }
        if (false === \seasar\container\Config::$SEARCH_PARENT_CONTAINER) {
            return null;
        }
        if (false === $searchParent) {
            return null;
        }
        $searchedAsParent[] = $this;
        if (count($this->parents) === 0) {
            return null;
        }
        foreach($this->parents as $parent) {
            if (in_array($parent, $searchedAsParent, true)) {
                continue;
            }
            $cd = $parent->getComponentDefRecursive($key, true,  $searchedAsParent, $searchedAsChild);
            if (!is_null($cd)) {
                return $cd;
            }
        }
        return null;
    }
    
    public function getComponentDefByNamespace($key) {
        $matches = array();
        if (preg_match('/(.+?)\.(.+)/', $key, $matches)) {
            $cd = $this->getComponentDefRecursive($matches[1], false);
            if (!is_null($cd)) {
                $childContainer = $cd->getComponent();
                if ($childContainer instanceof \seasar\container\S2Container) {
                    $cd = $cd->getComponent()->getComponentDefRecursive($matches[2], false);
                    if (!is_null($cd)) {
                        return $cd;
                    }
                }
            }
        }
        return null;
    }
    
    public function getComponentDefByChildren($key, &$searchedAsParent, &$searchedAsChild) {
        foreach ($this->children as $childContainer) {
            if (in_array($childContainer, $searchedAsParent, true)) {
                continue;
            }
            if (in_array($childContainer, $searchedAsChild, true)) {
                continue;
            }
            $cd = $childContainer->getComponentDefRecursive($key, false, $searchedAsParent, $searchedAsChild);
            if (!is_null($cd)) {
                return $cd;
            }
            $searchedAsChild[] = $childContainer;
        }
        return null;
    }
    
    public function hasComponentDef($componentKey) {
        return $this->getComponentDefInternal($componentKey) !== null;
    }
    
    public function hasDescendant($path) {
        return array_key_exists($path, $this->descendants);
    }
    
    public function getDescendant($path) {
        if ($this->hasDescendant($path)) {
            return $this->descendants[$path];
        } else {
            throw new \seasar\container\exception\ContainerNotRegisteredRuntimeException($path);
        }
    }
    
    public function registerDescendant(\seasar\container\S2Container $descendant) {
        $this->descendants[$descendant->getPath()] = $descendant;
    }
    
    public function includeChild(\seasar\container\S2Container $childContainer) {
        $childContainer->setRoot($this->getRoot());
        $childContainer->addParent($this);
        $this->children[] = $childContainer;
        $this->registerDescendant($childContainer);
        $ns = $childContainer->getNamespace();
        if ($ns !== null) {
            $this->registerMap($ns, new S2ContainerComponentDef($childContainer, $ns));
        }
    }
    
    public function getChildSize() {
        return count($this->children);
    }
    
    public function getChild($index) {
        if (!isset($this->children[$index])) {
            throw new \seasar\container\exception\ContainerNotRegisteredRuntimeException("Child:" . $index);
        }
        return $this->children[$index];
    }
    
    public function getNamespace() {
        return $this->namespace;
    }
    
    public function setNamespace($namespace) {
        $this->namespace = $namespace;
        $this->componentDefMap[$this->namespace] = new SimpleComponentDef($this, $this->namespace);
    }
    
    public function getPath() {
        return $this->path;
    }
    
    public function setPath($path) {
        $this->path = $path;
    }
    
    public function addMetaDef(MetaDef $metaDef) {
        $this->metaDefSupport->addMetaDef($metaDef);
    }
    
    public function getMetaDef($name) {
        return $this->metaDefSupport->getMetaDef($name);
    }
    
    public function getMetaDefs($name) {
        return $this->metaDefSupport->getMetaDefs($name);
    }
    
    public function getMetaDefSize() {
        return $this->metaDefSupport->getMetaDefSize();
    }
    
    private function getAssignableClasses(\ReflectionClass $componentClass) {
        $classes = array();
        $interfaces = \seasar\util\ClassUtil::getInterfaces($componentClass);
        foreach ($interfaces as $interface) {
            $classes[] = array($interface->getNamespaceName(), $interface->getShortName());
        }
        $reflection = $componentClass;
        if(!$reflection->isInterface()){
            while ($reflection instanceof \ReflectionClass) {
                $classes[] = array($reflection->getNamespaceName(), $reflection->getShortName());
                $reflection = $reflection->getParentClass();
            }
        }
        return $classes;
    }
    
    private function processTooManyRegistration($key, \seasar\container\ComponentDef $newComponentDef) {
        $componentDef = $this->componentDefMap[$key];
        if ($componentDef instanceof \seasar\container\impl\TooManyRegistrationComponentDef) {
            $componentDef->addComponentDef($newComponentDef);
        } else {
            $tmrcf = new \seasar\container\impl\TooManyRegistrationComponentDef($key);
            $tmrcf->addComponentDef($componentDef);
            $tmrcf->addComponentDef($newComponentDef);
            $this->componentDefMap[$key] = $tmrcf;
        }
    }
}

namespace seasar\container\util;
final class MetaDefSupport {
    
    private $metaDefs = array();
    
    private $container;
    
    public function __construct(\seasar\container\S2Container $container = null) {
        $this->coontainer_ = $container;
    }
    
    public function addMetaDef(\seasar\container\impl\MetaDef $metaDef) {
        if ($this->container != null) {
            $metaDef->setContainer($this->container);
        }
        $this->metaDefs[] = $metaDef;
    }
    
    public function getMetaDefSize() {
        return count($this->metaDefs);
    }
    
    public function getMetaDef($index) {
        if (is_integer($index)) {
            if (!isset($this->metaDefs[$index])) {
                throw new \OutOfRangeException($index);
            }
            return $this->metaDefs[$index];
        }
        foreach ($this->metaDefs as $metaDef) {
            if ($metaDef->getName() === $index) {
                return $metaDef;
            }
        }
        return null;
    }
    
    public function getMetaDefs($name) {
        $defs = array();
        foreach ($this->metaDefs as $def) {
            if ($def->getName() === $name) {
                $defs[] = $def;
            }
        }
        return $defs;
    }
    
    public function setContainer(\seasar\container\S2Container $container) {
        $this->container = $container;
        foreach ($this->metaDefs as $def) {
            $def->setContainer($container);
        }
    }
}

namespace seasar\container\impl;
class SimpleComponentDef implements \seasar\container\ComponentDef {
    
    private $component;
    
    private $componentClass;
    
    private $componentClassName;
    
    private $componentName;
    
    private $container;
    
    public function __construct($component, $componentName = '') {
        $this->component = $component;
        $this->componentClass = new \ReflectionClass($component);
        $this->componentClassName = $this->componentClass->getName();
        $this->componentName = $componentName;
    }
    
    public function getComponent() {
        return $this->component;
    }
    
    public function getContainer() {
        return $this->container;
    }
    
    public function setContainer(\seasar\container\S2Container $container) {
        $this->container = $container;
    }
    
    public function getComponentClass() {
        return $this->componentClass;
    }
    
    public function getComponentName() {
        return $this->componentName;
    }
    
    public final function setComponentName($name) {
        $this->componentName = $name;
    }
    
    
    public function addArgDef(\seasar\container\impl\ArgDef $argDef) {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function addPropertyDef(\seasar\container\impl\PropertyDef $propertyDef) {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function addInitMethodDef(\seasar\container\impl\InitMethodDef $methodDef) {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function addAspectDef(\seasar\container\impl\AspectDef $aspectDef) {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function getArgDefSize() {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function getPropertyDefSize() {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function getInitMethodDefSize() {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function getAspectDefSize() {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function getArgDefs() {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function getPropertyDefs() {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function getInitMethodDefs() {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function getAspectDefs() {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function getArgDef($index) {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function getPropertyDef($index) {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function hasPropertyDef($propertyName) {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function getInitMethodDef($index) {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function getAspectDef($index) {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function addMetaDef(\seasar\container\impl\MetaDef $metaDef) {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function getMetaDef($index) {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function getMetaDefs($name) {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function getMetaDefSize() {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function getInstanceDef() {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function setInstanceDef(\seasar\container\InstanceDef $instanceDef) {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function getAutoBindingDef() {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function setAutoBindingDef(\seasar\container\AutoBindingDef $autoBindingDef) {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function setConstructClosure(\Closure $closure) {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function getConstructClosure() {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
    
    public function hasConstructClosure() {
        throw new \seasar\exception\UnsupportedOperationException(__METHOD__);
    }
}

namespace seasar\container\factory;
class ComponentDefBuilder {
    
    public static $READ_PARENT_ANNOTATION = false;
    
    const COMPONENT_ANNOTATION = '@S2Component';
    
    const BINDING_ANNOTATION = '@S2Binding';
    
    const ASPECT_ANNOTATION = '@S2Aspect';
    
    const META_ANNOTATION = '@S2Meta';
    
    public static function create(\seasar\container\S2Container $container, \ReflectionClass $refClass) {
        $cd = self::createComponentDef($refClass);
        if ($cd instanceof \seasar\container\ComponentDef) {
            $cd->setContainer($container);
            self::setupComponentDef($cd);
        }
        return $cd;
    }
    
    public static function createComponentDef($refClass) {
        if (!\seasar\util\Annotation::has($refClass, self::COMPONENT_ANNOTATION)) {
            return new \seasar\container\impl\ComponentDefImpl($refClass);
        }
        $componentInfo = \seasar\util\Annotation::get($refClass, self::COMPONENT_ANNOTATION);
        if (isset($componentInfo['available']) and (boolean)$componentInfo['available'] === false) {
            return null;
        }
        if (isset($componentInfo['name'])) {
            $cd = new \seasar\container\impl\ComponentDefImpl($refClass, $componentInfo['name']);
        } else {
            $cd = new \seasar\container\impl\ComponentDefImpl($refClass);
        }
        if (isset($componentInfo['instance'])) {
            $cd->setInstanceDef(\seasar\container\deployer\InstanceDefFactory::getInstanceDef($componentInfo['instance']));
        }
        if (isset($componentInfo['autoBinding'])) {
            $cd->setAutoBindingDef(\seasar\container\assembler\AutoBindingDefFactory::getAutoBindingDef($componentInfo['autoBinding']));
        }
        return $cd;
    }
    
    public static function setupComponentDef(\seasar\container\ComponentDef $cd) {
        $classRef = $cd->getComponentClass();
        $beanDesc = \seasar\beans\BeanDescFactory::getBeanDesc($classRef);
        $propDescs = $beanDesc->getPropertyDescs();
        foreach ($propDescs as $propDesc) {
            $ref = $propDesc->getReflection();
            if (self::$READ_PARENT_ANNOTATION === false and 
                $ref->getDeclaringClass()->getName() !== $classRef->getName()) {
                continue;
            }
            if (\seasar\util\Annotation::has($ref, self::BINDING_ANNOTATION)) {
                self::setupPropertyDef($cd, $ref, $propDesc->getPropertyName());
            }
        }
        $methodRefs = $classRef->getMethods();
        foreach ($methodRefs as $methodRef) {
            if (self::$READ_PARENT_ANNOTATION === false and 
                $methodRef->getDeclaringClass()->getName() !== $classRef->getName()) {
                continue;
            }
            if (!$methodRef->isPublic() or 
                $methodRef->isConstructor() or
                strpos($methodRef->getName(), '_') === 0 ) {
                continue;
            }
            if (!$methodRef->isStatic() and 
                !$methodRef->isFinal() and
                \seasar\util\Annotation::has($methodRef, self::ASPECT_ANNOTATION)) {
                self::setupMethodAspectDef($cd, $methodRef);
            }
        }
        if (!$classRef->isFinal() and
            \seasar\util\Annotation::has($classRef, self::ASPECT_ANNOTATION)) {
            self::setupClassAspectDef($cd, $classRef);
        }
        if (\seasar\util\Annotation::has($cd->getComponentClass(), self::META_ANNOTATION)) {
            self::setupClassMetaDef($cd, $classRef);
        }
    }
    
    public static function setupPropertyDef(\seasar\container\ComponentDef $cd, $reflection, $propName) {
        $propInfo = \seasar\util\Annotation::get($reflection, self::BINDING_ANNOTATION);
        if (isset($propInfo[0])) {
            $propertyDef = new \seasar\container\impl\PropertyDef($propName);
            $cd->addPropertyDef($propertyDef);
            if ($cd->getContainer()->hasComponentDef($propInfo[0])) {
                $propertyDef->setChildComponentDef($cd->getContainer()->getComponentDef($propInfo[0]));
            } else {
                $propertyDef->setExpression($propInfo[0]);
            }
        } else {
            \seasar\log\S2Logger::getLogger(__CLASS__)->debug("binding annotation found. cannot get values.", __METHOD__);
        }
    }
    
    public static function setupClassAspectDef(\seasar\container\ComponentDef $cd, \ReflectionClass $classRef) {
        $annoInfo = \seasar\util\Annotation::get($classRef, self::ASPECT_ANNOTATION);
        if (count($annoInfo) === 0) {
            \seasar\log\S2Logger::getLogger(__CLASS__)->debug("class aspect annotation found. cannot get values.", __METHOD__);
            return;
        }
        if (array_key_exists(0, $annoInfo)) {
            $annoInfo['interceptor'] = $annoInfo[0];
        }
        self::setupAspectDef($cd, $annoInfo);
    }
    
    public static function setupMethodAspectDef(\seasar\container\ComponentDef $cd, \ReflectionMethod $methodRef) {
        $annoInfo = \seasar\util\Annotation::get($methodRef, self::ASPECT_ANNOTATION);
        if (count($annoInfo) === 0) {
            \seasar\log\S2Logger::getLogger(__CLASS__)->debug("method aspect annotation found. cannot get values.", __METHOD__);
            return;
        }
        if (array_key_exists(0, $annoInfo)) {
            $annoInfo['interceptor'] = $annoInfo[0];
        }
        $annoInfo['pointcut'] = '/^' . $methodRef->getName() . '$/';
        self::setupAspectDef($cd, $annoInfo);
    }
    
    public static function setupAspectDef(\seasar\container\ComponentDef $cd, array $annoInfo) {
        if (isset($annoInfo['interceptor'])) {
            if (isset($annoInfo['pointcut']) and is_string($annoInfo['pointcut'])) {
                $pointcut = new \seasar\aop\Pointcut($annoInfo['pointcut']);
            } else {
                $pointcut = new \seasar\aop\Pointcut($cd->getComponentClass());
            }
            $aspectDef = new \seasar\container\impl\AspectDef($pointcut);
            $cd->addAspectDef($aspectDef);
            if (is_string($annoInfo['interceptor'])) {
                if ($cd->getContainer()->hasComponentDef($annoInfo['interceptor'])) {
                    $aspectDef->setChildComponentDef($cd->getContainer()->getComponentDef($annoInfo['interceptor']));
                } else {
                    $aspectDef->setExpression($annoInfo['interceptor']);
                }
            } else {
                $aspectDef->setValue($annoInfo['interceptor']);
            }
        } else {
            \seasar\log\S2Logger::getLogger(__CLASS__)->debug("invalid aspect info. cannot get interceptor value.", __METHOD__);
        }
    }
    
    public static function setupClassMetaDef(\seasar\container\ComponentDef $cd, \ReflectionClass $classRef) {
        $annoInfo = \seasar\util\Annotation::get($classRef, self::META_ANNOTATION);
        if (count($annoInfo) === 0) {
            \seasar\log\S2Logger::getLogger(__CLASS__)->debug("class aspect annotation found. cannot get values.", __METHOD__);
            return;
        }
        foreach($annoInfo as $key => $val) {
            $metaDef = new \seasar\container\impl\MetaDef($key);
            $cd->addMetaDef($metaDef);
            $metaDef->setExpression($val);
            if ($cd->getContainer()->hasComponentDef($val)) {
                $metaDef->setChildComponentDef($cd->getContainer()->getComponentDef($val));
            } else {
                $metaDef->setExpression($val);
            }
        }
    }
}

namespace seasar\util;
class Annotation {
    
    public static $CONSTANT = true;
    
    public static $COMMENT  = true;
    
    private static $spool = array();
    
    private static $formattedComments = array();
    
    public static function readConstantAnnotation($val = true) {
        self::$CONSTANT = $val;
    }
    
    public static function readCommentAnnotation($val = true) {
        self::$COMMENT = $val;
    }
    
    public static function put($annoKey, array $annoArray) {
        self::$spool[$annoKey] = $annoArray;
    }
    
    public static function get($reflection, $annotation) {
        if (self::$CONSTANT and self::hasConstantAnnotation($reflection, $annotation)) {
            return self::getConstantAnnotation($reflection, $annotation);
        }
        if (self::$COMMENT) {
            return self::getCommentAnnotation($reflection, $annotation);
        }
        throw new \seasar\exception\AnnotationNotFoundException($reflection, $annotation);
    }
    
    public static function getConstantAnnotation($reflection, $key) {
        list($annotation, $atAnnotation) = self::constructAnnotationKey($key);
        self::validateReflection($reflection);
        $annoKey = self::constructKey($reflection) . '_' . $annotation;
        if (isset(self::$spool[$annoKey])) {
            return self::$spool[$annoKey];
        }
        if ($reflection instanceof \ReflectionProperty or
            $reflection instanceof \ReflectionMethod) {
            $annotation = $reflection->getName() . '_' . $annotation;
            $reflection = $reflection->getDeclaringClass();
        }
        $annoArray = null;
        if ($reflection->hasConstant($annotation)) {
            $value = EvalUtil::formatArrayExpression(trim($reflection->getConstant($annotation)));
            $annoArray = EvalUtil::execute($value);
        }
        if (is_array($annoArray)) {
            self::$spool[$annoKey] = $annoArray;
        } else {
            throw new \seasar\exception\AnnotationNotFoundException($reflection, $annotation);
        }
        return self::$spool[$annoKey];
    }
    
    public static function getCommentAnnotation($reflection, $key) {
        list($annotation, $atAnnotation) = self::constructAnnotationKey($key);
        self::validateReflection($reflection);
        $refKey  = self::constructKey($reflection);
        $annoKey = $refKey . '_' . $annotation;
        if (isset(self::$spool[$annoKey])) {
            return self::$spool[$annoKey];
        }
        if (!isset(self::$formattedComments[$refKey])) {
            self::$formattedComments[$refKey] = self::formatCommentLine($reflection->getDocComment());
        }
        $comment = self::$formattedComments[$refKey];
        $annoArray = null;
        $matches = array();
        if (preg_match("/$atAnnotation\s*\((.*?)\);/iu", $comment, $matches) or
            preg_match("/$atAnnotation\s*\((.*?)\)\s/iu", $comment, $matches)) {
            $value = EvalUtil::formatArrayExpression(trim($matches[1]));
            $annoArray = EvalUtil::execute($value);
        }
        if (is_array($annoArray)) {
            self::$spool[$annoKey] = $annoArray;
        } else {
            throw new \seasar\exception\AnnotationNotFoundException($reflection, $annotation);
        }
        return self::$spool[$annoKey];
    }
    
    public static function has($reflection, $annotation) {
        if (self::$CONSTANT) {
            $ret = self::hasConstantAnnotation($reflection, $annotation);
            if ($ret === true) {
                return true;
            } else if ($ret === false and self::$COMMENT === false) {
                return false;
            }
        }
        if (self::$COMMENT) {
            return self::hasCommentAnnotation($reflection, $annotation);
        }
        return false;
    }
    
    public static function hasConstantAnnotation($reflection, $annotation) {
        try {
            self::getConstantAnnotation($reflection, $annotation);
            return true;
        } catch (\seasar\exception\AnnotationNotFoundException $e) {
            return false;
        }
    }
    
    public static function hasCommentAnnotation($reflection, $annotation) {
        try {
            self::getCommentAnnotation($reflection, $annotation);
            return true;
        } catch (\seasar\exception\AnnotationNotFoundException $e) {
            return false;
        }
    }
    
    public static function formatCommentLine($commentLine) {
        $comments = preg_split('/[\n\r]/', $commentLine, -1, PREG_SPLIT_NO_EMPTY);
        $comment = ' ';
        $lineCount = count($comments);
        if ($lineCount == 0) {
            return '';
        } else if ($lineCount == 1) {
            $comment = substr(trim($comments[0]), 3, -2);
        } else {
            $atFound = false;
            $comment .= substr(trim($comments[0]), 3) . ' ';
            if (0 < strpos($comment, '@')) {
                $atFound = true;
            }
            for($i=1; $i<$lineCount-1; $i++) {
                if ($atFound === false) {
                    if (0 < strpos($comments[$i], '@')) {
                        $atFound = true;
                    }
                }
                if ($atFound === true) {
                    $comment .= substr(trim($comments[$i]), 1) . ' ';
                }
            }
            $last = substr(trim($comments[$lineCount-1]), 0, -2);
            if (0 < strlen($last)) {
                $comment .= substr($last, 1);
            }
        }
        return $comment;
    }
    
    public static function constructKey($reflection) {
        if ($reflection instanceof \ReflectionClass) {
            return $reflection->getName() . '::CLASS';
        } else if ($reflection instanceof \ReflectionMethod) {
            return $reflection->getDeclaringClass()->getName() . '::' . $reflection->getName() . '::METHOD';
        } else {
            return $reflection->getDeclaringClass()->getName() . '::' . $reflection->getName() . '::PROPERTY';
        }
    }
    
    protected static function validateReflection($reflection) {
        if (!$reflection instanceof \ReflectionClass and
            !$reflection instanceof \ReflectionMethod and
            !$reflection instanceof \ReflectionProperty) {
            throw new \seasar\exception\AnnotationNotSupportedException('unsupported reflection.');
        }
    }
    
    public static function constructAnnotationKey($key) {
        if (strpos($key, '@') === 0) {
            return array(substr($key, 1), $key);
        } else {
            return array($key, '@' . $key);
        }
    }
}

namespace seasar\exception;
class AnnotationNotFoundException extends \Exception {
    
    public function __construct($reflection, $annotation) {
        if ($reflection instanceof \ReflectionClass) {
            $target = 'class ' . $reflection->getName();
        } else {
            $target = $reflection->getDeclaringClass()->getName() . '->' . $reflection->getName();
        }
        parent::__construct("annotation [$annotation] not found on $target.");
    }
}

namespace seasar\container\impl;
class ComponentDefImpl implements \seasar\container\ComponentDef {
    
    private $componentClass = null;
    
    private $componentClassName = null;
    
    private $componentName = null;
    
    private $container = null;
    
    private $argDefSupport = null;
    
    private $propertyDefSupport = null;
    
    private $initMethodDefSupport = null;
    
    private $aspectDefSupport = null;
    
    private $metaDefSupport = null;
    
    private $instanceDef = null;
    
    private $autoBindingDef = null;
    
    private $componentDeployer = null;
    
    private $constructClosure = null;
    
    public function __construct($componentClassName, $componentName = null) {
        if ($componentClassName instanceof \ReflectionClass) {
            $componentClassName = $componentClassName->getName();
        }
        
        $this->componentClass       = new \ReflectionClass($componentClassName);
        $this->componentName        = $componentName;
        $this->argDefSupport        = new \seasar\container\util\ArgDefSupport();
        $this->propertyDefSupport   = new \seasar\container\util\PropertyDefSupport();
        $this->initMethodDefSupport = new \seasar\container\util\InitMethodDefSupport();
        $this->aspectDefSupport     = new \seasar\container\util\AspectDefSupport();
        $this->metaDefSupport       = new \seasar\container\util\MetaDefSupport();
        $this->instanceDef          = \seasar\container\deployer\InstanceDefFactory::getInstanceDef(\seasar\container\InstanceDef::SINGLETON_NAME);
        $this->autoBindingDef       = \seasar\container\assembler\AutoBindingDefFactory::getAutoBindingDef(\seasar\container\AutoBindingDef::AUTO_NAME);
    }
    
    public function getComponent() {
        return $this->getComponentDeployer()->deploy();
    }
    
    public final function getComponentClass() {
        return $this->componentClass;
    }
    
    public final function getComponentName() {
        return $this->componentName;
    }
    
    public final function setComponentName($name) {
        $this->componentName = $name;
    }
    
    public final function getContainer() {
        return $this->container;
    }
    
    public final function setContainer(\seasar\container\S2Container $container) {
        $this->container = $container;
        $this->argDefSupport->setContainer($container);
        $this->metaDefSupport->setContainer($container);
        $this->propertyDefSupport->setContainer($container);
        $this->initMethodDefSupport->setContainer($container);
        $this->aspectDefSupport->setContainer($container);
    }
    
    public function getInstanceDef() {
        return $this->instanceDef;
    }
    
    public function setInstanceDef(\seasar\container\InstanceDef $instanceDef) {
        $this->instanceDef = $instanceDef;
    }
    
    public function getAutoBindingDef() {
        return $this->autoBindingDef;
    }
    
    public function setAutoBindingDef(\seasar\container\AutoBindingDef $autoBindingDef) {
        $this->autoBindingDef = $autoBindingDef;
    }
    
    public function getArgDefs() {
        return $this->argDefSupport->getArgDefs();
    }
    
    public function getArgDef($index) {
        return $this->argDefSupport->getArgDef($index);
    }
    
    public function getArgDefSize() {
        return $this->argDefSupport->getArgDefSize();
    }
    
    public function addArgDef(\seasar\container\impl\ArgDef $argDef) {
        $this->argDefSupport->addArgDef($argDef);
    }
    
    public function getPropertyDefs() {
        return $this->propertyDefSupport->getPropertyDefs();
    }
    
    public function getPropertyDef($index) {
        return $this->propertyDefSupport->getPropertyDef($index);
    }
    
    public function hasPropertyDef($propertyName) {
        return $this->propertyDefSupport->hasPropertyDef($propertyName);
    }
    
    public function addPropertyDef(\seasar\container\impl\PropertyDef $propertyDef) {
        $this->propertyDefSupport->addPropertyDef($propertyDef);
    }
    
    public function getPropertyDefSize() {
        return $this->propertyDefSupport->getPropertyDefSize();
    }
    
    public function getInitMethodDefs() {
        return $this->initMethodDefSupport->getInitMethodDefs();
    }
    
    public function getInitMethodDef($index) {
        return $this->initMethodDefSupport->getInitMethodDef($index);
    }
    
    public function getInitMethodDefSize() {
        return $this->initMethodDefSupport->getInitMethodDefSize();
    }
    
    public function addInitMethodDef(\seasar\container\impl\InitMethodDef $methodDef) {
        $this->initMethodDefSupport->addInitMethodDef($methodDef);
    }
    
    public function getAspectDefs() {
        return $this->aspectDefSupport->getAspectDefs();
    }
    
    public function getAspectDef($index) {
        return $this->aspectDefSupport->getAspectDef($index);
    }
    
    public function getAspectDefSize() {
        return $this->aspectDefSupport->getAspectDefSize();
    }
    
    public function addAspectDef(\seasar\container\impl\AspectDef $aspectDef) {
        $this->aspectDefSupport->addAspectDef($aspectDef);
    }
    
    public function getMetaDef($index) {
        return $this->metaDefSupport->getMetaDef($index);
    }
    
    public function getMetaDefs($name) {
        return $this->metaDefSupport->getMetaDefs($name);
    }
    
    public function addMetaDef(\seasar\container\impl\MetaDef $metaDef) {
        $this->metaDefSupport->addMetaDef($metaDef);
    }
    
    public function getMetaDefSize() {
        return $this->metaDefSupport->getMetaDefSize();
    }
    
    private function getComponentDeployer() {
        if ($this->componentDeployer == null) {
            $this->componentDeployer = $this->instanceDef->createComponentDeployer($this);
        }
        return $this->componentDeployer;
    }
    
    public function setConstructClosure(\Closure $closure) {
        $this->constructClosure = $closure;
    }
    
    public function getConstructClosure() {
        return $this->constructClosure;
    }
    
    public function hasConstructClosure() {
        return !is_null($this->constructClosure);
    }
}

namespace seasar\container\util;
final class ArgDefSupport {
    
    private $argDefs = array();
    
    private $container = null;
    
    public function addArgDef(\seasar\container\impl\ArgDef $argDef) {
        if ($this->container != null) {
            $argDef->setContainer($this->container);
        }
        $this->argDefs[] = $argDef;
    }
    
    public function getArgDefSize() {
        return count($this->argDefs);
    }
    
    public function getArgDefs() {
        return $this->argDefs;
    }
    
    public function getArgDef($index) {
        if (!isset($this->argDefs[$index])) {
            throw new \OutOfRangeException($index);
        }
        return $this->argDefs[$index];
    }
    
    public function setContainer(\seasar\container\S2Container $container) {
        $this->container = $container;
        foreach ($this->argDefs as $argDef) {
            $argDef->setContainer($container);
        }
    }
}

namespace seasar\container\util;
final class PropertyDefSupport {
    
    private $propertyDefs    = array();
    
    private $propertyDefList = array();
    
    private $container       = null;
    
    public function addPropertyDef(\seasar\container\impl\PropertyDef $propertyDef) {
        if ($this->container != null) {
            $propertyDef->setContainer($this->container);
        }
        $this->propertyDefs[$propertyDef->getPropertyName()] = $propertyDef;
        $this->propertyDefList[] = $propertyDef->getPropertyName();
    }
    
    public function getPropertyDefs() {
        return $this->propertyDefs;
    }
    
    public function getPropertyDefSize() {
        return count($this->propertyDefs);
    }
    
    public function getPropertyDef($value) {
        if (is_integer($value)) {
            if (isset($this->propertyDefList[$value])) {
                return $this->propertyDefs[$this->propertyDefList[$value]];
            } else {
                throw new \OutOfRangeException($value);
            }
        }
        if (!$this->hasPropertyDef($value)) {
            throw new \OutOfRangeException($value);
        }
        return $this->propertyDefs[$value];
    }
    
    public function hasPropertyDef($propertyName) {
        return array_key_exists($propertyName, $this->propertyDefs);
    }
    
    public function setContainer(\seasar\container\S2Container $container) {
        $this->container = $container;
        foreach ($this->propertyDefs as $name => $def) {
            $def->setContainer($container);
        }
    }
}

namespace seasar\container\util;
class InitMethodDefSupport {
    
    private $methodDefs = array();
    
    private $container  = null;
    
    public function addInitMethodDef(\seasar\container\impl\InitMethodDef $methodDef) {
        if ($this->container !== null) {
            $methodDef->setContainer($this->container);
        }
        $this->methodDefs[] = $methodDef;
    }
    
    public function getInitMethodDefSize() {
        return count($this->methodDefs);
    }
    
    public function getInitMethodDef($index) {
        if (!isset($this->methodDefs[$index])) {
            throw new \OutOfRangeException($index);
        }
        return $this->methodDefs[$index];
    }
    
    public function getInitMethodDefs() {
        return $this->methodDefs;
    }
    
    public function setContainer(\seasar\container\S2Container $container) {
        $this->container = $container;
        foreach ($this->methodDefs as $methodDef) {
            $methodDef->setContainer($container);
        }
    }
}

namespace seasar\container\util;
final class AspectDefSupport {
    
    private $aspectDefs = array();
    
    private $container;
    
    public function addAspectDef(\seasar\container\impl\AspectDef $aspectDef) {
        if ($this->container !== null) {
            $aspectDef->setContainer($this->container);
        }
        $this->aspectDefs[] = $aspectDef;
    }
    
    public function getAspectDefSize() {
        return count($this->aspectDefs);
    }
    
    public function getAspectDefs() {
        return $this->aspectDefs;
    }
    
    public function getAspectDef($index) {
        if (!isset($this->aspectDefs[$index])) {
            throw new \OutOfRangeException($index);
        }
        return $this->aspectDefs[$index];
    }
    
    public function setContainer(\seasar\container\S2Container $container) {
        $this->container = $container;
        foreach ($this->aspectDefs as $def) {
            $def->setContainer($container);
        }
    }
}

namespace seasar\container\deployer;
class InstanceSingletonDef implements \seasar\container\InstanceDef {
    
    private $name;
    
    public function __construct($name) {
        $this->name = $name;
    }
    
    public function getName() {
        return $this->name;
    }
    
    public function createComponentDeployer(\seasar\container\ComponentDef $componentDef) {
        return new SingletonComponentDeployer($componentDef);
    }
}

namespace seasar\container\deployer;
class InstancePrototypeDef implements \seasar\container\InstanceDef {
    
    private $name;
    
    public function __construct($name) {
        $this->name = $name;
    }
    
    public function getName() {
        return $this->name;
    }
    
    public function createComponentDeployer(\seasar\container\ComponentDef $componentDef) {
        return new PrototypeComponentDeployer($componentDef);
    }
}

namespace seasar\container\deployer;
class InstanceDefFactory {
    
    private static $instanceDefs = array();
    
    private function __construct() {}
    
    public static function addInstanceDef(\seasar\container\InstanceDef $instanceDef) {
        self::$instanceDefs[$instanceDef->getName()] = $instanceDef;
    }
    
    public static function existInstanceDef($name) {
        return array_key_exists($name, self::$instanceDefs);
    }
    
    public static function getInstanceDef($name) {
        if (!self::existInstanceDef($name)) {
            throw new \seasar\container\exception\IllegalInstanceDefRuntimeException($name);
        }
        return self::$instanceDefs[$name];
    }
}

\seasar\container\deployer\InstanceDefFactory::addInstanceDef(new InstanceSingletonDef(\seasar\container\InstanceDef::SINGLETON_NAME));
\seasar\container\deployer\InstanceDefFactory::addInstanceDef(new InstancePrototypeDef(\seasar\container\InstanceDef::PROTOTYPE_NAME));

namespace seasar\container\assembler;
class AutoBindingAutoDef implements \seasar\container\AutoBindingDef {
    
    private $name = null;
    
    public function __construct($name) {
        $this->name = $name;
    }
    
    public function getName() {
        return $this->name;
    }
    
    public function createConstructorAssembler(\seasar\container\ComponentDef $componentDef) {
        return new AutoConstructorAssembler($componentDef);
    }
    
    public function createPropertyAssembler(\seasar\container\ComponentDef $componentDef) {
        return new AutoPropertyAssembler($componentDef);
    }
}

namespace seasar\container\assembler;
class AutoBindingNoneDef implements \seasar\container\AutoBindingDef {
    
    private $name = null;
    
    public function __construct($name) {
        $this->name = $name;
    }
    
    public function getName() {
        return $this->name;
    }
    
    public function createConstructorAssembler(\seasar\container\ComponentDef $componentDef) {
        return new ManualConstructorAssembler($componentDef);
    }
    
    public function createPropertyAssembler(\seasar\container\ComponentDef $componentDef) {
        return new ManualPropertyAssembler($componentDef);
    }
}

namespace seasar\container\assembler;
class AutoBindingDefFactory {
    
    private static $autoBindingDefs = array();
    
    private function __construct() {}
    
    public static function addAutoBindingDef(\seasar\container\AutoBindingDef $autoBindingDef) {
        self::$autoBindingDefs[$autoBindingDef->getName()] = $autoBindingDef;
    }
    
    public static function existAutoBindingDef($name) {
        return array_key_exists($name, self::$autoBindingDefs);
    }
    
    public static function getAutoBindingDef($name) {
        if (!self::existAutoBindingDef($name)) {
            throw new \seasar\container\exception\IllegalAutoBindingDefRuntimeException($name);
        }
        return self::$autoBindingDefs[$name];
    }
}

\seasar\container\assembler\AutoBindingDefFactory::addAutoBindingDef(new AutoBindingAutoDef(\seasar\container\AutoBindingDef::AUTO_NAME));
\seasar\container\assembler\AutoBindingDefFactory::addAutoBindingDef(new AutoBindingNoneDef(\seasar\container\AutoBindingDef::NONE_NAME));

namespace seasar\beans;
class BeanDescFactory {
    
    private static $spool = array();
    
    private function __construct() {}
    
    public static function create($obj) {
        if ($obj instanceof \ReflectionClass) {
            $clazz = $obj;
            $className = $clazz->getName();
        } else if (is_object($obj)) {
            $clazz = new \ReflectionClass($obj);
            $className = $clazz->getName();
        } else {
            $clazz = new \ReflectionClass($obj);
            $className = $obj;
        }
        if (!isset(self::$spool[$className])) {
            self::$spool[$className] = new BeanDesc($clazz);
        }
        return self::$spool[$className];
    }
    
    public static function getBeanDesc($value) {
        return self::create($value);
    }
}

namespace seasar\beans;
class BeanDesc {
    
    private $beanClass = null;
    
    private $propertyDescs = array();
    
    private $typehintPropertyDescs = array();
    
    public function __construct(\ReflectionClass $clazz) {
        $this->beanClass = $clazz;
        $this->setupPublicPropertyDescs();
        $this->setupAccessorMethodPropertyDescs();
    }
    
    public function getBeanClass(){
        return $this->beanClass;
    }
    
    public function getPropertyDescs() {
        return $this->propertyDescs;
    }
    
    public function getPropertyDescSize() {
        return count($this->propertyDescs);
    }
    
    public function hasPropertyDesc($name) {
        return array_key_exists($name, $this->propertyDescs);
    }
    
    public function getPropertyDesc($name) {
        if ($this->hasPropertyDesc($name)) {
            return $this->propertyDescs[$name];
        }
        throw new \seasar\exception\PropertyNotFoundRuntimeException($this->beanClass, $name);
    }
    
    public function getTypehintPropertyDescs() {
        return $this->typehintPropertyDescs;
    }
    
    public function getTypehintPropertyDescSize() {
        return count($this->typehintPropertyDescs);
    }
    
    public function hasTypehintPropertyDesc($name) {
        return array_key_exists($name, $this->typehintPropertyDescs);
    }
    
    public function getTypehintPropertyDesc($name) {
        if ($this->hasTypehintPropertyDesc($name)) {
            return $this->typehintPropertyDescs[$name];
        }
        throw new \seasar\exception\PropertyNotFoundRuntimeException($this->beanClass, $name);
    }
    
    private function setupPublicPropertyDescs() {
        $properties = $this->beanClass->getDefaultProperties();
        $propTypehintKey = \seasar\container\Config::$PROPERTY_TYPEHINT_KEY;
        $propTypehintKeyLen = strlen($propTypehintKey);
        foreach($properties as $name => $value) {
            if ($this->beanClass->getProperty($name)->isPublic()){
                $propertyDesc = new PublicPropertyDesc($this->beanClass, $name);
                $this->propertyDescs[$name] = $propertyDesc;
                if (true === \seasar\container\Config::$PROPERTY_TYPEHINT_NULL and
                    $value == null) {
                    $propertyDesc->setArrayAcceptable(false);
                    $propertyDesc->setTypehint($name);
                    $this->typehintPropertyDescs[$name] = $propertyDesc;
                } else if (is_string($value) and 
                    0 === stripos($value, $propTypehintKey)) {
                    $typehint = trim(substr($value, $propTypehintKeyLen));
                    if ($typehint === '') {
                        $typehint = $name;
                    }
                    if (0 === strpos(strrev($typehint), '][')) {
                        $typehint = substr($typehint, 0, -2);
                        if ($typehint === '') {
                            $typehint = $name;
                        }
                        $propertyDesc->setArrayAcceptable(true);
                    } else {
                        $propertyDesc->setArrayAcceptable(false);
                    }
                    if (class_exists($typehint)) {
                        $propertyDesc->setTypehint($typehint);
                        $propertyDesc->setTypehintClass(new \ReflectionClass($typehint));
                    } else {
                        $propertyDesc->setTypehint($typehint);
                    }
                    $this->typehintPropertyDescs[$name] = $propertyDesc;
                }
            }
        }
    }
    
    private function setupAccessorMethodPropertyDescs() {
        $methods = $this->beanClass->getMethods();
        foreach($methods as $method) {
            $params = $method->getParameters();
            $methodName = $method->getName();
            if ($method->isPublic() and
                count($params) === 1 and 
                0 === strpos($methodName, 'set')) {
                $propName = lcfirst(substr($methodName, 3));
                $propertyDesc = new AccessorMethodPropertyDesc($this->beanClass, $propName);
                $this->propertyDescs[$propName] = $propertyDesc;
                if ($params[0]->getClass() === null) {
                    $propertyDesc->setArrayAcceptable($params[0]->isArray());
                } else {
                    $propertyDesc->setTypehint($params[0]->getClass()->getName());
                    $propertyDesc->setTypehintClass($params[0]->getClass());
                    $this->typehintPropertyDescs[$propName] = $propertyDesc;
                }
            }
        }
    }
}

namespace seasar\util;
class StringUtil {
    
    private function __construct(){}
    
    public static function mixToString($val) {
        if (is_array($val)) {
            $c = count($val);
            return "array($c)";
        } else if (is_object($val)){
            $name = get_class($val);
            $id = spl_object_hash($val);
            return "object[$name#$id]";
        } else if (is_bool($val)){
            if ($val) {
                return 'true';
            } else {
                return 'false';
            }
        } else if (is_null($val)){
            return 'null';
        } else {
            return (string)$val;
        }
    }
}

namespace seasar\beans;
abstract class AbstractPropertyDesc implements PropertyDesc {
    
    protected $typehint = null;
    
    protected $typehintClass = null;
    
    protected $isArrayAcceptable = false;
    
    protected $beanClass = null;
    
    protected $propertyName = null;
    
    public function __construct(\ReflectionClass $beanClass, $propName) {
        $this->beanClass = $beanClass;
        $this->propertyName = $propName;
    }
    
    public function getBeanClass() {
        return $this->beanClass;
    }
    
    public function getPropertyName() {
        return $this->propertyName;
    }
    
    public function getTypehint() {
        return $this->typehint;
    }
    
    public function setTypehint($value) {
        $this->typehint = $value;
    }
    
    public function getTypehintClass() {
        return $this->typehintClass;
    }
    
    public function setTypehintClass(\ReflectionClass $clazz) {
        $this->typehintClass = $clazz;
    }
    
    public function isArrayAcceptable() {
        return $this->isArrayAcceptable;
    }
    
    public function setArrayAcceptable($value = true) {
        $this->isArrayAcceptable = $value;
    }
}

namespace seasar\beans;
class AccessorMethodPropertyDesc extends AbstractPropertyDesc {
    
    private $writeMethod = null;
    
    private $readMethod = null;
    
    public function __construct(\ReflectionClass $beanClass, $propName) {
        parent::__construct($beanClass, $propName);
        $methodName = self::getWriteMethodName($propName);
        if ($beanClass->hasMethod($methodName)) {
            $method = $beanClass->getMethod($methodName);
            if ($method->isPublic()) {
                $this->writeMethod = $method;
            }
        }
        $methodName = self::getReadMethodName($propName);
        if ($beanClass->hasMethod($methodName)) {
            $method = $beanClass->getMethod($methodName);
            if ($method->isPublic()) {
                $this->readMethod = $method;
            }
        }
        if ($this->writeMethod === null and $this->readMethod === null) {
            throw new \seasar\exception\IllegalPropertyRuntimeException($propName);
        }
    }
    
    public static function getWriteMethodName($propName) {
        return 'set' . ucfirst($propName);
    }
    
    public static function getReadMethodName($propName) {
        return 'get' . ucfirst($propName);
    }
    
    public function getWriteMethod() {
        return $this->writeMethod;
    }
    
    public function setWriteMethod(\ReflectionMethod $method) {
        $this->writeMethod = $method;
    }
    
    public function getReadMethod() {
        return $this->readMethod;
    }
    
    public function setReadMethod(\ReflectionMethod $method) {
        $this->readMethod = $method;
    }
    
    public function hasReadMethod() {
        return $this->readMethod instanceof \ReflectionMethod;
    }
    
    public function hasWriteMethod() {
        return $this->writeMethod instanceof \ReflectionMethod;
    }
    
    public function isReadable() {
        return $this->readMethod instanceof \ReflectionMethod;
    }
    
    public function isWritable() {
        return $this->writeMethod instanceof \ReflectionMethod;
    }
    
    public function setValue($instance, $value) {
        if ($this->writeMethod === null) {
            throw new \seasar\exception\MethodNotFoundRuntimeException(self::getWriteMethodName($this->propertyName));
        }
        $this->writeMethod->invoke($instance, $value);
    }
    
    public function getValue($instance) {
        if ($this->readMethod === null) {
            throw new \seasar\exception\MethodNotFoundRuntimeException(self::getReadMethodName($this->propertyName));
        }
        return $this->readMethod->invoke($instance);
    }
    
    public function getReflection() {
        return $this->writeMethod;
    }
}

namespace seasar\util;
final class ClassUtil {
    
    private function __construct() {}
    
    public static function getInterfaces(\ReflectionClass $clazz) {
        $interfaces = array_values($clazz->getInterfaces());
        if ($clazz->isInterface()) {
            $interfaces[] = $clazz;
        }
        return $interfaces;
    }
    
    public static function getDeclaringMethods(\ReflectionClass $clazz) {
        if ($clazz->getParentClass() === null) {
            return $clazz->getMethods();
        }
        $implMethods = $clazz->getMethods();
        $methods = array();
        $className = $clazz->getName();
        foreach($implMethods as $implMethod) {
            if (!$implMethod->isConstructor() and
                $implMethod->getDeclaringClass()->getName() === $className) {
                $methods[] = $implMethod;
            }
        }
        return $methods;
    }
    
    public static function getImplementMethods(\ReflectionClass $clazz) {
        if ($clazz->isInterface()) {
            return $clazz->getMethods();
        }
        $methods = array();
        if ($clazz->isAbstract()) {
            $methodRefs = $clazz->getMethods();
            foreach ($methodRefs as $methodRef) {
                if($methodRef->isAbstract()){
                    $methods[] = $methodRef;
                }
            }
        }
        $interfaces = self::getInterfaces($clazz);
        foreach ($interfaces as $interface) {
            $methods = array_merge($methods, $interface->getMethods());
        }
        return $methods;
    }
    
    public static function getAbstractMethods(\ReflectionClass $clazz) {
        if ($clazz->isInterface()) {
            return $clazz->getMethods();
        }
        $methods = array();
        if ($clazz->isAbstract()) {
            $methodRefs = $clazz->getMethods();
            foreach ($methodRefs as $methodRef) {
                if($methodRef->isAbstract()){
                    $methods[] = $methodRef;
                }
            }
        }
        return $methods;
    }
    
    public static function newInstance(\ReflectionClass $reflection, array $args = array()) {
        if (count($args) > 0) {
            return $reflection->newInstanceArgs($args);
        }
        return $reflection->newInstance();
    }
}

namespace seasar\container\deployer;
abstract class AbstractComponentDeployer {
    
    private $componentDef         = null;
    
    private $constructorAssembler = null;
    
    private $propertyAssembler    = null;
    
    private $initMethodAssembler  = null;
    
    public function __construct(\seasar\container\ComponentDef $componentDef) {
        $this->componentDef = $componentDef;
        $this->setupAssembler();
    }
    
    abstract public function deploy();
    
    protected final function getComponentDef() {
        return $this->componentDef;
    }
    
    protected final function getConstructorAssembler() {
        return $this->constructorAssembler;
    }
    
    protected final function getPropertyAssembler() {
        return $this->propertyAssembler;
    }
    
    protected final function getInitMethodAssembler() {
        return $this->initMethodAssembler;
    }
    
    protected function setupAssembler() {
        $this->constructorAssembler = $this->componentDef->getAutoBindingDef()->createConstructorAssembler($this->componentDef);
        $this->propertyAssembler    = $this->componentDef->getAutoBindingDef()->createPropertyAssembler($this->componentDef);
        $this->initMethodAssembler  = new \seasar\container\assembler\InitMethodAssembler($this->componentDef);
    }
}

namespace seasar\container\deployer;
class SingletonComponentDeployer extends AbstractComponentDeployer {
    
    private $component     = null;
    
    private $instantiating = false;
    
    public function __construct(\seasar\container\ComponentDef $componentDef) {
        parent::__construct($componentDef);
    }
    
    public function deploy() {
        if ($this->component === null) {
            if ($this->instantiating) {
                throw new \seasar\container\exception\CyclicReferenceRuntimeException($this->getComponentDef()->getComponentClass());
            }
            $this->instantiating = true;
            $this->component     = $this->getConstructorAssembler()->assemble();
            $this->instantiating = false;
            $this->getPropertyAssembler()->assemble($this->component);
            $this->getInitMethodAssembler()->assemble($this->component);
        }
        return $this->component;
    }
}

namespace seasar\container\assembler;
abstract class AbstractAssembler {
    
    private $componentDef;
    
    public function __construct(\seasar\container\ComponentDef $componentDef) {
        $this->componentDef = $componentDef;
    }
    
    protected final function getComponentDef() {
        return $this->componentDef;
    }
    
    protected function getArgs(array $propRefs) {
        $args = array();
        $container = $this->getComponentDef()->getContainer();
        foreach($propRefs as $propRef) {
            if ($propRef->getClass() !== null and
                $container->hasComponentDef($propRef->getClass()->getName())) {
                $args[] = $container->getComponent($propRef->getClass()->getName());
            } else {
                if ($propRef->isOptional()) {
                    $args[] = $propRef->getDefaultValue();
                } else {
                    $args[] = null;
                }
            }
        }
        return $args;
    }
    
    protected function getArgument(\seasar\container\impl\ArgDef $argDef, $isArrayAcceptable = false) {
        $value  = null;
        try {
            $value = $argDef->getValue();
            if ($isArrayAcceptable and !is_array($value)) {
                $value = array($value);
            }
        } catch(\seasar\container\exception\TooManyRegistrationRuntimeException $e) {
            if ($isArrayAcceptable) {
                $childComponentDefs = $argDef->getChildComponentDef()->getComponentDefs();
                $value = array();
                foreach ($childComponentDefs as $childComponentDef) {
                    $value[] = $childComponentDef->getComponent();
                }
            } else {
                throw $e;
            }
        }
        return $value;
    }
}

namespace seasar\container\assembler;
class ConstructClosureAssembler extends AbstractAssembler {
    
    public function assemble() {
        $componentDef = $this->getComponentDef();
        if ($componentDef->hasConstructClosure()) {
            $componentClass = $componentDef->getComponentClass();
            $closure = $componentDef->getConstructClosure();
            $component = $closure($componentDef);
            if (is_object($component) && $componentClass->isInstance($component)) {
                return $component;
            } else {
                if (is_object($component)) { 
                    $realClassName = get_class($component);
                } else if(is_null($component)) {
                    $realClassName = 'null';
                } else {
                    $realClassName = strval($component);
                }
                throw new \seasar\container\exception\ClassUnmatchRuntimeException($componentClass->getName(), $realClassName);
            }
        } else {
            return $componentDef->getComponentClass()->newInstance();
        }
    }
}

namespace seasar\container\assembler;
class ManualConstructorAssembler extends ConstructClosureAssembler {
    
    public function assemble() {
        $componentDef   = $this->getComponentDef();
        if ($componentDef->hasConstructClosure()) {
            return parent::assemble();
        }
        $args           = array();
        $refParams      = array();
        $refConstructor = $componentDef->getComponentClass()->getConstructor();
        if ($refConstructor instanceof \ReflectionMethod) {
            $refParams = $refConstructor->getParameters();
        }
        $argDefs = $componentDef->getArgDefs();
        $o = count($argDefs);
        for ($i = 0; $i < $o; ++$i) {
            if (isset($refParams[$i]) and $refParams[$i]->isArray()) {
                $args[] = $this->getArgument($argDefs[$i], true);
            } else {
                $args[] = $this->getArgument($argDefs[$i], false);
            }
        }
        return \seasar\container\util\ConstructorUtil::newInstance($componentDef, $args);
    }
}

namespace seasar\container\assembler;
class AutoConstructorAssembler extends ManualConstructorAssembler {
    
    public function assemble() {
        $componentDef = $this->getComponentDef();
        if ($componentDef->hasConstructClosure() || $componentDef->getArgDefSize() > 0) {
            return parent::assemble();
        }
        $args = array();
        $refMethod = $componentDef->getComponentClass()->getConstructor();
        if ($refMethod instanceof \ReflectionMethod) {
            $args = $this->getArgs($refMethod->getParameters());
        }
        return \seasar\container\util\ConstructorUtil::newInstance($componentDef, $args);
    }
}

namespace seasar\container\assembler;
class ManualPropertyAssembler extends AbstractAssembler {
    
    public function __construct(\seasar\container\ComponentDef $componentDef) {
        parent::__construct($componentDef);
    }
    
    public function assemble($component) {
        $componentDef = $this->getComponentDef();
        $beanDesc = \seasar\beans\BeanDescFactory::getBeanDesc($componentDef->getComponentClass());
        $propDescs = $beanDesc->getTypehintPropertyDescs();
        $propertyDefs = $componentDef->getPropertyDefs();
        foreach ($propertyDefs as $propertyName => $propertyDef) {
            $value = null;
            if ($beanDesc->hasPropertyDesc($propertyName)) {
                $propDesc = $beanDesc->getPropertyDesc($propertyName);
                try {
                    $value = $propertyDef->getValue();
                    if ($propDesc->isArrayAcceptable() and !is_array($value)) {
                        $value = array($value);
                    }
                } catch(\seasar\container\exception\TooManyRegistrationRuntimeException $e) {
                    if ($propDesc->isArrayAcceptable()) {
                        $childComponentDefs = $propertyDef->getChildComponentDef()->getComponentDefs();
                        $value = array();
                        foreach ($childComponentDefs as $childComponentDef) {
                           $value[] = $childComponentDef->getComponent();
                        }
                    } else {
                        throw $e;
                    }
                }
                $propDesc->setValue($component, $value);
            } else {
                throw new \seasar\exception\PropertyNotFoundRuntimeException($componentDef->getComponentClass(), $propertyName);
            }
        }
    }
}

namespace seasar\container\assembler;
class AutoPropertyAssembler extends ManualPropertyAssembler {
    
    public function __construct(\seasar\container\ComponentDef $componentDef) {
        parent::__construct($componentDef);
    }
    
    public function assemble($component) {
        parent::assemble($component);
        $componentDef = $this->getComponentDef();
        $container    = $componentDef->getContainer();
        $beanDesc = \seasar\beans\BeanDescFactory::getBeanDesc($componentDef->getComponentClass());
        $propDescs = $beanDesc->getTypehintPropertyDescs();
        foreach ($propDescs as $propDesc) {
            if ($componentDef->hasPropertyDef($propDesc->getPropertyName())) {
                continue;
            }
            $value   = null;
            try {
                if ($propDesc->isArrayAcceptable()) {
                    $value = $container->findComponents($propDesc->getTypehint());
                } else {
                    $value = $container->getComponent($propDesc->getTypehint());
                }
            } catch (\seasar\container\exception\ComponentNotFoundRuntimeException $e) {
                \seasar\log\S2Logger::getInstance(__CLASS__)->info("no component found for typehint property. [{$componentDef->getComponentClass()->getName()}::\${$propDesc->getPropertyName()}]", __METHOD__);
                continue;
            } catch (\seasar\container\exception\TooManyRegistrationRuntimeException $e) {
                \seasar\log\S2Logger::getInstance(__CLASS__)->info("too many component found for typehint property. ignored. [{$componentDef->getComponentClass()->getName()}::\${$propDesc->getPropertyName()}]", __METHOD__);
                continue;
            }
            $propDesc->setValue($component, $value);
        }
    }
}

namespace seasar\container\assembler;
abstract class AbstractMethodAssembler extends AbstractAssembler {
    
    public function __construct(\seasar\container\ComponentDef $componentDef) {
        parent::__construct($componentDef);
    }
    
    protected function invoke($component, \seasar\container\MethodDef $methodDef) {
        if ($methodDef->getMethodName() == null) {
            \seasar\util\EvalUtil::execute($methodDef->getExpression(), array('component' => $component));
        } else {
            $refMethod = $this->getComponentDef()->getComponentClass()->getMethod($methodDef->getMethodName());
            $refParams = $refMethod->getParameters();
            $args = array();
            $argDefs = $methodDef->getArgDefs();
            $o = count($argDefs);
            if (0 < $o) {
                for ($i = 0; $i < $o; ++$i) {
                    if (isset($refParams[$i]) and $refParams[$i]->isArray()) {
                        $args[] = $this->getArgument($argDefs[$i], true);
                    } else {
                        $args[] = $this->getArgument($argDefs[$i], false);
                    }
                }
            } else {
                $args = $this->getArgs($refParams);
            }
            \seasar\util\MethodUtil::invoke($refMethod, $component, $args);
        }
    }
}

namespace seasar\container\assembler;
class InitMethodAssembler extends AbstractMethodAssembler {
    
    public function __construct(\seasar\container\ComponentDef $componentDef) {
        parent::__construct($componentDef);
    }
    
    public function assemble($component) {
        $initMethodDefs = $this->getComponentDef()->getInitMethodDefs();
        foreach ($initMethodDefs as $initMethodDef) {
            $this->invoke($component, $initMethodDef);
        }
    }
}

namespace seasar\container\util;
final class ConstructorUtil {
    
    private function __construct() {}
    
    public static function newInstance(\seasar\container\ComponentDef $componentDef, array $args = array()) {
        if ($componentDef->getAspectDefSize() > 0) {
            return \seasar\container\util\AopUtil::getInstance($componentDef, $args);
        }
        $reflection = $componentDef->getComponentClass();
        if ($reflection->getConstructor() instanceof \ReflectionMethod) {
            return $reflection->newInstanceArgs($args);
        }
        if (count($args) > 0) {
            throw new \seasar\container\exception\IllegalConstructorRuntimeException($reflection);
        }
        return $reflection->newInstance();
    }
    
    public static function getInstance(\seasar\container\ComponentDef $componentDef, array $args = array()) {
        return self::newInstance($componentDef, $args);
    }
}
